@@ -18,18 +18,21 @@ IRONIC_INSPECTOR_URI="http://$IRONIC_INSPECTOR_HOST:$IRONIC_INSPECTOR_PORT"
 IRONIC_INSPECTOR_BUILD_RAMDISK=$(trueorfalse False IRONIC_INSPECTOR_BUILD_RAMDISK)
 IRONIC_AGENT_KERNEL_URL=${IRONIC_AGENT_KERNEL_URL:-http://tarballs.openstack.org/ironic-python-agent/coreos/files/coreos_production_pxe.vmlinuz}
 IRONIC_AGENT_RAMDISK_URL=${IRONIC_AGENT_RAMDISK_URL:-http://tarballs.openstack.org/ironic-python-agent/coreos/files/coreos_production_pxe_image-oem.cpio.gz}
-IRONIC_INSPECTOR_COLLECTORS=${IRONIC_INSPECTOR_COLLECTORS:-default,logs}
+IRONIC_INSPECTOR_COLLECTORS=${IRONIC_INSPECTOR_COLLECTORS:-default,logs,pci-devices}
 IRONIC_INSPECTOR_RAMDISK_LOGDIR=${IRONIC_INSPECTOR_RAMDISK_LOGDIR:-$IRONIC_INSPECTOR_DATA_DIR/ramdisk-logs}
 IRONIC_INSPECTOR_ALWAYS_STORE_RAMDISK_LOGS=${IRONIC_INSPECTOR_ALWAYS_STORE_RAMDISK_LOGS:-True}
 IRONIC_INSPECTOR_TIMEOUT=${IRONIC_INSPECTOR_TIMEOUT:-600}
+IRONIC_INSPECTOR_CLEAN_UP_PERIOD=${IRONIC_INSPECTOR_CLEAN_UP_PERIOD:-}
 # These should not overlap with other ranges/networks
 IRONIC_INSPECTOR_INTERNAL_IP=${IRONIC_INSPECTOR_INTERNAL_IP:-172.24.42.254}
 IRONIC_INSPECTOR_INTERNAL_SUBNET_SIZE=${IRONIC_INSPECTOR_INTERNAL_SUBNET_SIZE:-24}
 IRONIC_INSPECTOR_DHCP_RANGE=${IRONIC_INSPECTOR_DHCP_RANGE:-172.24.42.100,172.24.42.253}
 IRONIC_INSPECTOR_INTERFACE=${IRONIC_INSPECTOR_INTERFACE:-br-inspector}
+IRONIC_INSPECTOR_INTERFACE_PHYSICAL=$(trueorfalse False IRONIC_INSPECTOR_INTERFACE_PHYSICAL)
 IRONIC_INSPECTOR_INTERNAL_URI="http://$IRONIC_INSPECTOR_INTERNAL_IP:$IRONIC_INSPECTOR_PORT"
 IRONIC_INSPECTOR_INTERNAL_IP_WITH_NET="$IRONIC_INSPECTOR_INTERNAL_IP/$IRONIC_INSPECTOR_INTERNAL_SUBNET_SIZE"
-
+# Whether DevStack will be setup for bare metal or VMs
+IRONIC_IS_HARDWARE=$(trueorfalse False IRONIC_IS_HARDWARE)
 IRONIC_INSPECTOR_NODE_NOT_FOUND_HOOK=${IRONIC_INSPECTOR_NODE_NOT_FOUND_HOOK:-""}
 
 GITDIR["python-ironic-inspector-client"]=$DEST/python-ironic-inspector-client
@@ -53,8 +56,6 @@ function inspector_iniset {
 
 function install_inspector {
     setup_develop $IRONIC_INSPECTOR_DIR
-    # NOTE(dtantsur): required for tests
-    install_package jq
 }
 
 function install_inspector_dhcp {
@@ -66,8 +67,7 @@ function install_inspector_client {
         git_clone_by_name python-ironic-inspector-client
         setup_dev_lib python-ironic-inspector-client
     else
-        # TODO(dtantsur): switch to pip_install_gr
-        pip_install python-ironic-inspector-client
+        pip_install_gr python-ironic-inspector-client
     fi
 }
 
@@ -107,9 +107,9 @@ function prepare_tftp {
     else
         # download the agent image tarball
         if [ ! -e "$IRONIC_INSPECTOR_KERNEL_PATH" -o ! -e "$IRONIC_INSPECTOR_INITRAMFS_PATH" ]; then
-            if [ -e "$IRONIC_DEPLOY_KERNEL_PATH" -a -e "$IRONIC_DEPLOY_RAMDISK_PATH" ]; then
-                cp $IRONIC_DEPLOY_KERNEL_PATH $IRONIC_INSPECTOR_KERNEL_PATH
-                cp $IRONIC_DEPLOY_RAMDISK_PATH $IRONIC_INSPECTOR_INITRAMFS_PATH
+            if [ -e "$IRONIC_DEPLOY_KERNEL" -a -e "$IRONIC_DEPLOY_RAMDISK" ]; then
+                cp $IRONIC_DEPLOY_KERNEL $IRONIC_INSPECTOR_KERNEL_PATH
+                cp $IRONIC_DEPLOY_RAMDISK $IRONIC_INSPECTOR_INITRAMFS_PATH
             else
                 wget "$IRONIC_AGENT_KERNEL_URL" -O $IRONIC_INSPECTOR_KERNEL_PATH
                 wget "$IRONIC_AGENT_RAMDISK_URL" -O $IRONIC_INSPECTOR_INITRAMFS_PATH
@@ -203,12 +203,13 @@ function configure_inspector {
     mkdir_chown_stack "$IRONIC_INSPECTOR_RAMDISK_LOGDIR"
     inspector_iniset processing ramdisk_logs_dir "$IRONIC_INSPECTOR_RAMDISK_LOGDIR"
     inspector_iniset processing always_store_ramdisk_logs "$IRONIC_INSPECTOR_ALWAYS_STORE_RAMDISK_LOGS"
-    inspector_iniset processing log_bmc_address False
     if [ -n "$IRONIC_INSPECTOR_NODE_NOT_FOUND_HOOK" ]; then
         inspector_iniset processing node_not_found_hook "$IRONIC_INSPECTOR_NODE_NOT_FOUND_HOOK"
     fi
     inspector_iniset DEFAULT timeout $IRONIC_INSPECTOR_TIMEOUT
-
+    if [ -n "$IRONIC_INSPECTOR_CLEAN_UP_PERIOD" ]; then
+        inspector_iniset DEFAULT clean_up_period "$IRONIC_INSPECTOR_CLEAN_UP_PERIOD"
+    fi
     get_or_create_service "ironic-inspector" "baremetal-introspection" "Ironic Inspector baremetal introspection service"
     get_or_create_endpoint "baremetal-introspection" "$REGION_NAME" \
         "$IRONIC_INSPECTOR_URI" "$IRONIC_INSPECTOR_URI" "$IRONIC_INSPECTOR_URI"
@@ -252,10 +253,12 @@ function prepare_environment {
     prepare_tftp
     create_ironic_inspector_cache_dir
 
-    sudo ip link add brbm-inspector type veth peer name $IRONIC_INSPECTOR_INTERFACE
-    sudo ip link set dev brbm-inspector up
+    if [[ "$IRONIC_IS_HARDWARE" == "False" ]]; then
+        sudo ip link add brbm-inspector type veth peer name $IRONIC_INSPECTOR_INTERFACE
+        sudo ip link set dev brbm-inspector up
+        sudo ovs-vsctl add-port brbm brbm-inspector
+    fi
     sudo ip link set dev $IRONIC_INSPECTOR_INTERFACE up
-    sudo ovs-vsctl add-port brbm brbm-inspector
     sudo ip addr add $IRONIC_INSPECTOR_INTERNAL_IP_WITH_NET dev $IRONIC_INSPECTOR_INTERFACE
 
     sudo iptables -I INPUT -i $IRONIC_INSPECTOR_INTERFACE -p udp \
@@ -274,10 +277,11 @@ function create_ironic_inspector_cache_dir {
 }
 
 function cleanup_inspector {
-    rm -f $IRONIC_TFTPBOOT_DIR/pxelinux.cfg/default
-    rm -f $IRONIC_TFTPBOOT_DIR/ironic-inspector.*
     if [[ "$IRONIC_IPXE_ENABLED" == "True" ]] ; then
         rm -f $IRONIC_HTTP_DIR/ironic-inspector.*
+    else
+        rm -f $IRONIC_TFTPBOOT_DIR/pxelinux.cfg/default
+        rm -f $IRONIC_TFTPBOOT_DIR/ironic-inspector.*
     fi
     sudo rm -f /etc/sudoers.d/ironic-inspector-rootwrap
     sudo rm -rf $IRONIC_INSPECTOR_AUTH_CACHE_DIR
@@ -293,7 +297,9 @@ function cleanup_inspector {
     sudo iptables -F ironic-inspector | true
     sudo iptables -X ironic-inspector | true
 
-    sudo ip link show $IRONIC_INSPECTOR_INTERFACE && sudo ip link delete $IRONIC_INSPECTOR_INTERFACE
+    if [[ $IRONIC_INSPECTOR_INTERFACE != $OVS_PHYSICAL_BRIDGE && "$IRONIC_INSPECTOR_INTERFACE_PHYSICAL" == "False" ]]; then
+        sudo ip link show $IRONIC_INSPECTOR_INTERFACE && sudo ip link delete $IRONIC_INSPECTOR_INTERFACE
+    fi
     sudo ip link show brbm-inspector && sudo ip link delete brbm-inspector
     sudo ovs-vsctl --if-exists del-port brbm-inspector
 }
@@ -327,6 +333,13 @@ elif [[ "$1" == "stack" && "$2" == "extra" ]]; then
         start_inspector_dhcp
     fi
     start_inspector
+elif [[ "$1" == "stack" && "$2" == "test-config" ]]; then
+    if is_service_enabled tempest; then
+        echo_summary "Configuring Tempest for Ironic Inspector"
+        if [ -n "$IRONIC_INSPECTOR_NODE_NOT_FOUND_HOOK" ]; then
+            iniset $TEMPEST_CONFIG baremetal_introspection auto_discovery_feature True
+        fi
+    fi
 fi
 
 if [[ "$1" == "unstack" ]]; then
