@@ -22,12 +22,13 @@ from oslo_db import options as db_opts
 from oslo_db.sqlalchemy import models
 from oslo_db.sqlalchemy import session as db_session
 from oslo_db.sqlalchemy import types as db_types
-from sqlalchemy import (Boolean, Column, DateTime, Float, ForeignKey, Integer,
-                        String, Text)
+from sqlalchemy import (Boolean, Column, DateTime, Enum, ForeignKey,
+                        Integer, String, Text)
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy import orm
 
 from ironic_inspector import conf  # noqa
+from ironic_inspector import introspection_state as istate
 
 
 class ModelBase(models.ModelBase):
@@ -42,25 +43,35 @@ _FACADE = None
 
 db_opts.set_defaults(cfg.CONF, _DEFAULT_SQL_CONNECTION,
                      'ironic_inspector.sqlite')
-if CONF.discoverd.database:
-    db_opts.set_defaults(CONF,
-                         connection='sqlite:///%s' %
-                         str(CONF.discoverd.database).strip())
 
 
 class Node(Base):
     __tablename__ = 'nodes'
     uuid = Column(String(36), primary_key=True)
-    started_at = Column(Float, nullable=True)
-    finished_at = Column(Float, nullable=True)
+    version_id = Column(String(36), server_default='')
+    state = Column(Enum(*istate.States.all()), nullable=False,
+                   default=istate.States.finished,
+                   server_default=istate.States.finished)
+    started_at = Column(DateTime, nullable=True)
+    finished_at = Column(DateTime, nullable=True)
     error = Column(Text, nullable=True)
 
+    # version_id is being tracked in the NodeInfo object
+    # for the sake of consistency. See also SQLAlchemy docs:
+    # http://docs.sqlalchemy.org/en/latest/orm/versioning.html
+    __mapper_args__ = {
+        'version_id_col': version_id,
+        'version_id_generator': False,
+    }
+
 
 class Attribute(Base):
     __tablename__ = 'attributes'
-    name = Column(String(255), primary_key=True)
-    value = Column(String(255), primary_key=True)
-    uuid = Column(String(36), ForeignKey('nodes.uuid'))
+    uuid = Column(String(36), primary_key=True)
+    node_uuid = Column(String(36), ForeignKey('nodes.uuid',
+                                              name='fk_node_attribute'))
+    name = Column(String(255), nullable=False)
+    value = Column(String(255), nullable=True)
 
 
 class Option(Base):
@@ -92,6 +103,7 @@ class RuleCondition(Base):
     rule = Column(String(36), ForeignKey('rules.uuid'))
     op = Column(String(255), nullable=False)
     multiple = Column(String(255), nullable=False)
+    invert = Column(Boolean, default=False)
     # NOTE(dtantsur): while all operations now require a field, I can also
     # imagine user-defined operations that do not, thus it's nullable.
     field = Column(Text)
