@@ -27,6 +27,7 @@ from oslo_log import log
 from oslo_utils import uuidutils
 import werkzeug
 
+from ironic_inspector import api_tools
 from ironic_inspector import db
 from ironic_inspector.common.i18n import _, _LC, _LE, _LI, _LW
 from ironic_inspector.common import ironic as ir_utils
@@ -47,7 +48,10 @@ app = flask.Flask(__name__)
 LOG = utils.getProcessingLogger(__name__)
 
 MINIMUM_API_VERSION = (1, 0)
-CURRENT_API_VERSION = (1, 6)
+# TODO(dtantsur): set to the current version as soon we move setting IPMI
+# credentials support completely.
+DEFAULT_API_VERSION = (1, 8)
+CURRENT_API_VERSION = (1, 9)
 _LOGGING_EXCLUDED_KEYS = ('logs',)
 
 
@@ -66,7 +70,7 @@ def _format_version(ver):
     return '%d.%d' % ver
 
 
-_DEFAULT_API_VERSION = _format_version(CURRENT_API_VERSION)
+_DEFAULT_API_VERSION = _format_version(DEFAULT_API_VERSION)
 
 
 def error_response(exc, code=500):
@@ -133,6 +137,26 @@ def generate_resource_data(resources):
     return data
 
 
+def generate_introspection_status(node):
+    """Return a dict representing current node status.
+
+    :param node: a NodeInfo instance
+    :return: dictionary
+    """
+    started_at = node.started_at.isoformat()
+    finished_at = node.finished_at.isoformat() if node.finished_at else None
+
+    status = {}
+    status['uuid'] = node.uuid
+    status['finished'] = bool(node.finished_at)
+    status['started_at'] = started_at
+    status['finished_at'] = finished_at
+    status['error'] = node.error
+    status['links'] = create_link_object(
+        ["v%s/introspection/%s" % (CURRENT_API_VERSION[0], node.uuid)])
+    return status
+
+
 @app.route('/', methods=['GET'])
 @convert_exceptions
 def api_root():
@@ -200,14 +224,33 @@ def api_introspection(node_id):
         else:
             new_ipmi_credentials = None
 
+        if new_ipmi_credentials and _get_version() >= (1, 9):
+            return _('Setting IPMI credentials is deprecated and not allowed '
+                     'starting with API version 1.9'), 400
+
         introspect.introspect(node_id,
                               new_ipmi_credentials=new_ipmi_credentials,
                               token=flask.request.headers.get('X-Auth-Token'))
         return '', 202
     else:
         node_info = node_cache.get_node(node_id)
-        return flask.json.jsonify(finished=bool(node_info.finished_at),
-                                  error=node_info.error or None)
+        return flask.json.jsonify(generate_introspection_status(node_info))
+
+
+@app.route('/v1/introspection', methods=['GET'])
+@convert_exceptions
+def api_introspection_statuses():
+    utils.check_auth(flask.request)
+
+    nodes = node_cache.get_node_list(
+        marker=api_tools.marker_field(),
+        limit=api_tools.limit_field(default=CONF.api_max_limit)
+    )
+    data = {
+        'introspection': [generate_introspection_status(node)
+                          for node in nodes]
+    }
+    return flask.json.jsonify(data)
 
 
 @app.route('/v1/introspection/<node_id>/abort', methods=['POST'])
@@ -308,8 +351,6 @@ def handle_404(error):
     return error_response(error, code=404)
 
 
-@periodics.periodic(spacing=CONF.firewall.firewall_update_period,
-                    enabled=CONF.firewall.manage_firewall)
 def periodic_update():  # pragma: no cover
     try:
         firewall.update_filters()
@@ -317,7 +358,6 @@ def periodic_update():  # pragma: no cover
         LOG.exception(_LE('Periodic update of firewall rules failed'))
 
 
-@periodics.periodic(spacing=CONF.clean_up_period)
 def periodic_clean_up():  # pragma: no cover
     try:
         if node_cache.clean_up():
@@ -389,7 +429,7 @@ class Service(object):
         CONF.log_opt_values(LOG, log.DEBUG)
 
     def init(self):
-        if utils.get_auth_strategy() != 'noauth':
+        if CONF.auth_strategy != 'noauth':
             utils.add_auth_middleware(app)
         else:
             LOG.warning(_LW('Starting unauthenticated, please check'
@@ -411,8 +451,9 @@ class Service(object):
             hooks = [ext.name for ext in
                      plugins_base.processing_hooks_manager()]
         except KeyError as exc:
-            # stevedore raises KeyError on missing hook
-            LOG.critical(_LC('Hook %s failed to load or was not found'),
+            # callback function raises MissingHookError derived from KeyError
+            # on missing hook
+            LOG.critical(_LC('Hook(s) %s failed to load or was not found'),
                          str(exc))
             sys.exit(1)
 
@@ -421,9 +462,17 @@ class Service(object):
         if CONF.firewall.manage_firewall:
             firewall.init()
 
+        periodic_update_ = periodics.periodic(
+            spacing=CONF.firewall.firewall_update_period,
+            enabled=CONF.firewall.manage_firewall
+        )(periodic_update)
+        periodic_clean_up_ = periodics.periodic(
+            spacing=CONF.clean_up_period
+        )(periodic_clean_up)
+
         self._periodics_worker = periodics.PeriodicWorker(
-            callables=[(periodic_update, None, None),
-                       (periodic_clean_up, None, None)],
+            callables=[(periodic_update_, None, None),
+                       (periodic_clean_up_, None, None)],
             executor_factory=periodics.ExistingExecutor(utils.executor()))
         utils.executor().submit(self._periodics_worker.start)
 
