@@ -15,8 +15,10 @@
 
 import sys
 
+from ironic_lib import utils as il_utils
 import netaddr
 from oslo_config import cfg
+from oslo_utils import netutils
 from oslo_utils import units
 import six
 
@@ -46,42 +48,27 @@ class RootDiskSelectionHook(base.ProcessingHook):
                       node_info=node_info, data=introspection_data)
             return
 
-        inventory = introspection_data.get('inventory')
-        if not inventory:
+        inventory = utils.get_inventory(introspection_data,
+                                        node_info=node_info)
+        try:
+            device = il_utils.match_root_device_hints(inventory['disks'],
+                                                      hints)
+        except (TypeError, ValueError) as e:
             raise utils.Error(
-                _('Root device selection requires ironic-python-agent '
-                  'as an inspection ramdisk'),
+                _('No disks could be found using the root device hints '
+                  '%(hints)s because they failed to validate. '
+                  'Error: %(error)s') % {'hints': hints, 'error': e},
                 node_info=node_info, data=introspection_data)
 
-        disks = inventory.get('disks', [])
-        if not disks:
-            raise utils.Error(_('No disks found'),
+        if not device:
+            raise utils.Error(_('No disks satisfied root device hints'),
                               node_info=node_info, data=introspection_data)
 
-        for disk in disks:
-            properties = disk.copy()
-            # Root device hints are in GiB, data from IPA is in bytes
-            properties['size'] //= units.Gi
-
-            for name, value in hints.items():
-                actual = properties.get(name)
-                if actual != value:
-                    LOG.debug('Disk %(disk)s does not satisfy hint '
-                              '%(name)s=%(value)s, actual value is %(actual)s',
-                              {'disk': disk.get('name'), 'name': name,
-                               'value': value, 'actual': actual},
-                              node_info=node_info, data=introspection_data)
-                    break
-            else:
-                LOG.debug('Disk %(disk)s of size %(size)s satisfies '
-                          'root device hints',
-                          {'disk': disk.get('name'), 'size': disk['size']},
-                          node_info=node_info, data=introspection_data)
-                introspection_data['root_disk'] = disk
-                return
-
-        raise utils.Error(_('No disks satisfied root device hints'),
-                          node_info=node_info, data=introspection_data)
+        LOG.debug('Disk %(disk)s of size %(size)s satisfies '
+                  'root device hints',
+                  {'disk': device.get('name'), 'size': device['size']},
+                  node_info=node_info, data=introspection_data)
+        introspection_data['root_disk'] = device
 
 
 class SchedulerHook(base.ProcessingHook):
@@ -91,7 +78,8 @@ class SchedulerHook(base.ProcessingHook):
 
     def before_update(self, introspection_data, node_info, **kwargs):
         """Update node with scheduler properties."""
-        inventory = introspection_data.get('inventory')
+        inventory = utils.get_inventory(introspection_data,
+                                        node_info=node_info)
         errors = []
 
         root_disk = introspection_data.get('root_disk')
@@ -99,40 +87,24 @@ class SchedulerHook(base.ProcessingHook):
             introspection_data['local_gb'] = root_disk['size'] // units.Gi
             if CONF.processing.disk_partitioning_spacing:
                 introspection_data['local_gb'] -= 1
-        elif inventory:
-            errors.append(_('root disk is not supplied by the ramdisk and '
-                            'root_disk_selection hook is not enabled'))
-
-        if inventory:
-            try:
-                introspection_data['cpus'] = int(inventory['cpu']['count'])
-                introspection_data['cpu_arch'] = six.text_type(
-                    inventory['cpu']['architecture'])
-            except (KeyError, ValueError, TypeError):
-                errors.append(_('malformed or missing CPU information: %s') %
-                              inventory.get('cpu'))
-
-            try:
-                introspection_data['memory_mb'] = int(
-                    inventory['memory']['physical_mb'])
-            except (KeyError, ValueError, TypeError):
-                errors.append(_('malformed or missing memory information: %s; '
-                                'introspection requires physical memory size '
-                                'from dmidecode') %
-                              inventory.get('memory'))
         else:
-            LOG.warning(_LW('No inventory provided: using old bash ramdisk '
-                            'is deprecated, please switch to '
-                            'ironic-python-agent'),
-                        node_info=node_info, data=introspection_data)
+            introspection_data['local_gb'] = 0
 
-            missing = [key for key in self.KEYS
-                       if not introspection_data.get(key)]
-            if missing:
-                raise utils.Error(
-                    _('The following required parameters are missing: %s') %
-                    missing,
-                    node_info=node_info, data=introspection_data)
+        try:
+            introspection_data['cpus'] = int(inventory['cpu']['count'])
+            introspection_data['cpu_arch'] = six.text_type(
+                inventory['cpu']['architecture'])
+        except (KeyError, ValueError, TypeError):
+            errors.append(_('malformed or missing CPU information: %s') %
+                          inventory.get('cpu'))
+
+        try:
+            introspection_data['memory_mb'] = int(
+                inventory['memory']['physical_mb'])
+        except (KeyError, ValueError, TypeError):
+            errors.append(_('malformed or missing memory information: %s; '
+                            'introspection requires physical memory size '
+                            'from dmidecode') % inventory.get('memory'))
 
         if errors:
             raise utils.Error(_('The following problems encountered: %s') %
@@ -175,42 +147,38 @@ class ValidateInterfacesHook(base.ProcessingHook):
         :return: dict interface name -> dict with keys 'mac' and 'ip'
         """
         result = {}
-        inventory = data.get('inventory', {})
+        inventory = utils.get_inventory(data)
 
-        if inventory:
-            for iface in inventory.get('interfaces', ()):
-                name = iface.get('name')
-                mac = iface.get('mac_address')
-                ip = iface.get('ipv4_address')
+        for iface in inventory['interfaces']:
+            name = iface.get('name')
+            mac = iface.get('mac_address')
+            ip = iface.get('ipv4_address')
+            client_id = iface.get('client_id')
 
-                if not name:
-                    LOG.error(_LE('Malformed interface record: %s'),
-                              iface, data=data)
-                    continue
+            if not name:
+                LOG.error(_LE('Malformed interface record: %s'),
+                          iface, data=data)
+                continue
 
-                if not mac:
-                    LOG.debug('Skipping interface %s without link information',
-                              name, data=data)
-                    continue
+            if not mac:
+                LOG.debug('Skipping interface %s without link information',
+                          name, data=data)
+                continue
 
-                if not utils.is_valid_mac(mac):
-                    LOG.warning(_LW('MAC %(mac)s for interface %(name)s is '
-                                    'not valid, skipping'),
-                                {'mac': mac, 'name': name},
-                                data=data)
-                    continue
+            if not netutils.is_valid_mac(mac):
+                LOG.warning(_LW('MAC %(mac)s for interface %(name)s is '
+                                'not valid, skipping'),
+                            {'mac': mac, 'name': name},
+                            data=data)
+                continue
 
-                mac = mac.lower()
+            mac = mac.lower()
 
-                LOG.debug('Found interface %(name)s with MAC "%(mac)s" and '
-                          'IP address "%(ip)s"',
-                          {'name': name, 'mac': mac, 'ip': ip}, data=data)
-                result[name] = {'ip': ip, 'mac': mac}
-        else:
-            LOG.warning(_LW('No inventory provided: using old bash ramdisk '
-                            'is deprecated, please switch to '
-                            'ironic-python-agent'), data=data)
-            result = data.get('interfaces')
+            LOG.debug('Found interface %(name)s with MAC "%(mac)s", '
+                      'IP address "%(ip)s" and client_id "%(client_id)s"',
+                      {'name': name, 'mac': mac, 'ip': ip,
+                       'client_id': client_id}, data=data)
+            result[name] = {'ip': ip, 'mac': mac, 'client_id': client_id}
 
         return result
 
@@ -233,6 +201,7 @@ class ValidateInterfacesHook(base.ProcessingHook):
         for name, iface in interfaces.items():
             mac = iface.get('mac')
             ip = iface.get('ip')
+            client_id = iface.get('client_id')
 
             if name == 'lo' or (ip and netaddr.IPAddress(ip).is_loopback()):
                 LOG.debug('Skipping local interface %s', name, data=data)
@@ -249,7 +218,8 @@ class ValidateInterfacesHook(base.ProcessingHook):
                           name, data=data)
                 continue
 
-            result[name] = {'ip': ip, 'mac': mac.lower()}
+            result[name] = {'ip': ip, 'mac': mac.lower(),
+                            'client_id': client_id}
 
         if not result:
             raise utils.Error(_('No suitable interfaces found in %s') %
