@@ -11,6 +11,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import datetime
 
 import futurist
 import mock
@@ -26,6 +27,7 @@ from ironic_inspector.common import i18n
 # Import configuration options
 from ironic_inspector import conf  # noqa
 from ironic_inspector import db
+from ironic_inspector import introspection_state as istate
 from ironic_inspector import node_cache
 from ironic_inspector.plugins import base as plugins_base
 from ironic_inspector import utils
@@ -87,21 +89,37 @@ class InventoryTest(BaseTest):
         # Prepare some realistic inventory
         # https://github.com/openstack/ironic-inspector/blob/master/HTTP-API.rst  # noqa
         self.bmc_address = '1.2.3.4'
-        self.macs = ['11:22:33:44:55:66', '66:55:44:33:22:11']
-        self.ips = ['1.2.1.2', '1.2.1.1']
+        self.macs = (
+            ['11:22:33:44:55:66', '66:55:44:33:22:11', '7c:fe:90:29:26:52'])
+        self.ips = ['1.2.1.2', '1.2.1.1', '1.2.1.3']
         self.inactive_mac = '12:12:21:12:21:12'
         self.pxe_mac = self.macs[0]
         self.all_macs = self.macs + [self.inactive_mac]
         self.pxe_iface_name = 'eth1'
+        self.client_id = (
+            'ff:00:00:00:00:00:02:00:00:02:c9:00:7c:fe:90:03:00:29:26:52')
+        self.valid_interfaces = {
+            self.pxe_iface_name: {'ip': self.ips[0], 'mac': self.macs[0],
+                                  'client_id': None},
+            'ib0': {'ip': self.ips[2], 'mac': self.macs[2],
+                    'client_id': self.client_id}
+        }
         self.data = {
             'boot_interface': '01-' + self.pxe_mac.replace(':', '-'),
             'inventory': {
                 'interfaces': [
                     {'name': 'eth1', 'mac_address': self.macs[0],
-                     'ipv4_address': self.ips[0]},
+                     'ipv4_address': self.ips[0],
+                     'lldp': [
+                         [1, "04112233aabbcc"],
+                         [2, "07373334"],
+                         [3, "003c"]]},
                     {'name': 'eth2', 'mac_address': self.inactive_mac},
                     {'name': 'eth3', 'mac_address': self.macs[1],
                      'ipv4_address': self.ips[1]},
+                    {'name': 'ib0', 'mac_address': self.macs[2],
+                     'ipv4_address': self.ips[2],
+                     'client_id': self.client_id}
                 ],
                 'disks': [
                     {'name': '/dev/sda', 'model': 'Big Data Disk',
@@ -121,16 +139,25 @@ class InventoryTest(BaseTest):
             'root_disk': {'name': '/dev/sda', 'model': 'Big Data Disk',
                           'size': 1000 * units.Gi,
                           'wwn': None},
+            'interfaces': self.valid_interfaces,
         }
         self.inventory = self.data['inventory']
         self.all_interfaces = {
-            'eth1': {'mac': self.macs[0], 'ip': self.ips[0]},
-            'eth2': {'mac': self.inactive_mac, 'ip': None},
-            'eth3': {'mac': self.macs[1], 'ip': self.ips[1]}
+            'eth1': {'mac': self.macs[0], 'ip': self.ips[0],
+                     'client_id': None},
+            'eth2': {'mac': self.inactive_mac, 'ip': None, 'client_id': None},
+            'eth3': {'mac': self.macs[1], 'ip': self.ips[1],
+                     'client_id': None},
+            'ib0': {'mac': self.macs[2], 'ip': self.ips[2],
+                    'client_id': self.client_id}
         }
         self.active_interfaces = {
-            'eth1': {'mac': self.macs[0], 'ip': self.ips[0]},
-            'eth3': {'mac': self.macs[1], 'ip': self.ips[1]}
+            'eth1': {'mac': self.macs[0], 'ip': self.ips[0],
+                     'client_id': None},
+            'eth3': {'mac': self.macs[1], 'ip': self.ips[1],
+                     'client_id': None},
+            'ib0': {'mac': self.macs[2], 'ip': self.ips[2],
+                    'client_id': self.client_id}
         }
         self.pxe_interfaces = {
             self.pxe_iface_name: self.all_interfaces[self.pxe_iface_name]
@@ -158,6 +185,22 @@ class NodeTest(InventoryTest):
         self.node.to_dict = mock_to_dict
 
         self.ports = []
-        self.node_info = node_cache.NodeInfo(uuid=self.uuid, started_at=0,
-                                             node=self.node, ports=self.ports)
+        self.node_info = node_cache.NodeInfo(
+            uuid=self.uuid,
+            started_at=datetime.datetime(1, 1, 1),
+            node=self.node, ports=self.ports)
         self.node_info.node = mock.Mock(return_value=self.node)
+
+
+class NodeStateTest(NodeTest):
+    def setUp(self):
+        super(NodeStateTest, self).setUp()
+        self.node_info._version_id = uuidutils.generate_uuid()
+        self.node_info._state = istate.States.starting
+        self.db_node = db.Node(uuid=self.node_info.uuid,
+                               version_id=self.node_info._version_id,
+                               state=self.node_info._state,
+                               started_at=self.node_info.started_at,
+                               finished_at=self.node_info.finished_at,
+                               error=self.node_info.error)
+        self.db_node.save(self.session)
