@@ -11,6 +11,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import datetime
 import json
 import ssl
 import sys
@@ -70,6 +71,15 @@ class TestApiIntrospect(BaseAPITest):
             new_ipmi_credentials=('user', 'password'),
             token=None)
 
+    @mock.patch.object(introspect, 'introspect', autospec=True)
+    def test_introspect_set_ipmi_credentials_disabled(self, introspect_mock):
+        headers = {conf.VERSION_HEADER: '1.9'}
+        res = self.app.post('/v1/introspection/%s?new_ipmi_username=user&'
+                            'new_ipmi_password=password' % self.uuid,
+                            headers=headers)
+        self.assertEqual(400, res.status_code)
+        self.assertFalse(introspect_mock.called)
+
     @mock.patch.object(introspect, 'introspect', autospec=True)
     def test_introspect_set_ipmi_credentials_no_user(self, introspect_mock):
         res = self.app.post('/v1/introspection/%s?'
@@ -175,28 +185,93 @@ class TestApiAbort(BaseAPITest):
         self.assertEqual(str(exc), data['error']['message'])
 
 
-class TestApiGetStatus(BaseAPITest):
-    @mock.patch.object(node_cache, 'get_node', autospec=True)
+class GetStatusAPIBaseTest(BaseAPITest):
+    def setUp(self):
+        super(GetStatusAPIBaseTest, self).setUp()
+        self.uuid2 = uuidutils.generate_uuid()
+        self.finished_node = node_cache.NodeInfo(
+            uuid=self.uuid,
+            started_at=datetime.datetime(1, 1, 1),
+            finished_at=datetime.datetime(1, 1, 2),
+            error='boom')
+        self.finished_node.links = [
+            {u'href': u'http://localhost/v1/introspection/%s' %
+             self.finished_node.uuid,
+             u'rel': u'self'},
+        ]
+        self.finished_node.status = {
+            'finished': True,
+            'started_at': self.finished_node.started_at.isoformat(),
+            'finished_at': self.finished_node.finished_at.isoformat(),
+            'error': self.finished_node.error,
+            'uuid': self.finished_node.uuid,
+            'links': self.finished_node.links
+        }
+
+        self.unfinished_node = node_cache.NodeInfo(
+            uuid=self.uuid2,
+            started_at=datetime.datetime(1, 1, 1))
+        self.unfinished_node.links = [
+            {u'href': u'http://localhost/v1/introspection/%s' %
+             self.unfinished_node.uuid,
+             u'rel': u'self'}
+        ]
+        finished_at = (self.unfinished_node.finished_at.isoformat()
+                       if self.unfinished_node.finished_at else None)
+        self.unfinished_node.status = {
+            'finished': False,
+            'started_at': self.unfinished_node.started_at.isoformat(),
+            'finished_at': finished_at,
+            'error': None,
+            'uuid': self.unfinished_node.uuid,
+            'links': self.unfinished_node.links
+        }
+
+
+@mock.patch.object(node_cache, 'get_node', autospec=True)
+class TestApiGetStatus(GetStatusAPIBaseTest):
     def test_get_introspection_in_progress(self, get_mock):
-        get_mock.return_value = node_cache.NodeInfo(uuid=self.uuid,
-                                                    started_at=42.0)
+        get_mock.return_value = self.unfinished_node
         res = self.app.get('/v1/introspection/%s' % self.uuid)
         self.assertEqual(200, res.status_code)
-        self.assertEqual({'finished': False, 'error': None},
+        self.assertEqual(self.unfinished_node.status,
                          json.loads(res.data.decode('utf-8')))
 
-    @mock.patch.object(node_cache, 'get_node', autospec=True)
     def test_get_introspection_finished(self, get_mock):
-        get_mock.return_value = node_cache.NodeInfo(uuid=self.uuid,
-                                                    started_at=42.0,
-                                                    finished_at=100.1,
-                                                    error='boom')
+        get_mock.return_value = self.finished_node
         res = self.app.get('/v1/introspection/%s' % self.uuid)
         self.assertEqual(200, res.status_code)
-        self.assertEqual({'finished': True, 'error': 'boom'},
+        self.assertEqual(self.finished_node.status,
                          json.loads(res.data.decode('utf-8')))
 
 
+@mock.patch.object(node_cache, 'get_node_list', autospec=True)
+class TestApiListStatus(GetStatusAPIBaseTest):
+
+    def test_list_introspection(self, list_mock):
+        list_mock.return_value = [self.finished_node, self.unfinished_node]
+        res = self.app.get('/v1/introspection')
+        self.assertEqual(200, res.status_code)
+        statuses = json.loads(res.data.decode('utf-8')).get('introspection')
+
+        self.assertEqual([self.finished_node.status,
+                          self.unfinished_node.status], statuses)
+        list_mock.assert_called_once_with(marker=None,
+                                          limit=CONF.api_max_limit)
+
+    def test_list_introspection_limit(self, list_mock):
+        res = self.app.get('/v1/introspection?limit=1000')
+        self.assertEqual(200, res.status_code)
+        list_mock.assert_called_once_with(marker=None, limit=1000)
+
+    def test_list_introspection_makrer(self, list_mock):
+        res = self.app.get('/v1/introspection?marker=%s' %
+                           self.finished_node.uuid)
+        self.assertEqual(200, res.status_code)
+        list_mock.assert_called_once_with(marker=self.finished_node.uuid,
+                                          limit=CONF.api_max_limit)
+
+
 class TestApiGetData(BaseAPITest):
     @mock.patch.object(main.swift, 'SwiftAPI', autospec=True)
     def test_get_introspection_data(self, swift_mock):
@@ -510,7 +585,7 @@ class TestApiVersions(BaseAPITest):
                     u'href': u'http://localhost/v1/endpoint3'}]
             },
         ]}
-        self.assertDictEqual(expected, json_data)
+        self.assertEqual(expected, json_data)
 
     def test_version_endpoint_invalid(self):
         endpoint = "/v-1"
@@ -584,6 +659,7 @@ class TestInit(test_base.BaseTest):
     @mock.patch.object(firewall, 'clean_up', lambda: None)
     def tearDown(self):
         self.service.shutdown()
+        super(TestInit, self).tearDown()
 
     def test_ok(self, mock_node_cache, mock_get_client, mock_auth,
                 mock_firewall):
@@ -626,7 +702,6 @@ class TestInit(test_base.BaseTest):
         self.service.init()
         self.assertFalse(mock_firewall.called)
 
-    @unittest.skip('skipped until stevedore > 1.15.0 is released')
     @mock.patch.object(main.LOG, 'critical')
     def test_init_failed_processing_hook(self, mock_log, mock_node_cache,
                                          mock_get_client, mock_auth,
