@@ -11,17 +11,18 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import datetime
 import logging as pylog
-import re
 
 import futurist
 from keystonemiddleware import auth_token
 from oslo_config import cfg
 from oslo_log import log
 from oslo_middleware import cors as cors_middleware
-import six
+import pytz
 
-from ironic_inspector.common.i18n import _, _LE
+from ironicclient.v1 import node
+from ironic_inspector.common.i18n import _, _LE, _LI
 from ironic_inspector import conf  # noqa
 
 CONF = cfg.CONF
@@ -49,7 +50,7 @@ def processing_logger_prefix(data=None, node_info=None):
     """Calculate prefix for logging.
 
     Tries to use:
-    * node UUID,
+    * node UUID, node._state
     * node PXE MAC,
     * node BMC address
 
@@ -62,7 +63,10 @@ def processing_logger_prefix(data=None, node_info=None):
     data = data or {}
 
     if node_info is not None:
-        parts.append(str(node_info.uuid))
+        if isinstance(node_info, node.Node):
+            parts.append(str(node_info.uuid))
+        else:
+            parts.append(str(node_info))
 
     pxe_mac = get_pxe_mac(data)
     if pxe_mac:
@@ -113,9 +117,22 @@ class Error(Exception):
 class NotFoundInCacheError(Error):
     """Exception when node was not found in cache during processing."""
 
-    def __init__(self, msg, code=404):
+    def __init__(self, msg, code=404, **kwargs):
         super(NotFoundInCacheError, self).__init__(msg, code,
-                                                   log_level='info')
+                                                   log_level='info', **kwargs)
+
+
+class NodeStateRaceCondition(Error):
+    """State mismatch between the DB and a node_info."""
+    def __init__(self, *args, **kwargs):
+        message = _('Node state mismatch detected between the DB and the '
+                    'cached node_info object')
+        kwargs.setdefault('code', 500)
+        super(NodeStateRaceCondition, self).__init__(message, *args, **kwargs)
+
+
+class NodeStateInvalidEvent(Error):
+    """Invalid event attempted."""
 
 
 def executor():
@@ -133,29 +150,6 @@ def add_auth_middleware(app):
     :param app: application.
     """
     auth_conf = dict(CONF.keystone_authtoken)
-    # These items should only be used for accessing Ironic API.
-    # For keystonemiddleware's authentication,
-    # keystone_authtoken's items will be used and
-    # these items will be unsupported.
-    # [ironic]/os_password
-    # [ironic]/os_username
-    # [ironic]/os_auth_url
-    # [ironic]/os_tenant_name
-    auth_conf.update({'admin_password':
-                      CONF.ironic.os_password or
-                      CONF.keystone_authtoken.admin_password,
-                      'admin_user':
-                      CONF.ironic.os_username or
-                      CONF.keystone_authtoken.admin_user,
-                      'auth_uri':
-                      CONF.ironic.os_auth_url or
-                      CONF.keystone_authtoken.auth_uri,
-                      'admin_tenant_name':
-                      CONF.ironic.os_tenant_name or
-                      CONF.keystone_authtoken.admin_tenant_name,
-                      'identity_uri':
-                      CONF.ironic.identity_uri or
-                      CONF.keystone_authtoken.identity_uri})
     auth_conf['delay_auth_decision'] = True
     app.wsgi_app = auth_token.AuthProtocol(app.wsgi_app, auth_conf)
 
@@ -177,7 +171,7 @@ def check_auth(request):
     :param request: Flask request
     :raises: utils.Error if access is denied
     """
-    if get_auth_strategy() == 'noauth':
+    if CONF.auth_strategy == 'noauth':
         return
     if request.headers.get('X-Identity-Status').lower() == 'invalid':
         raise Error(_('Authentication required'), code=401)
@@ -187,21 +181,46 @@ def check_auth(request):
         raise Error(_('Access denied'), code=403)
 
 
-def is_valid_mac(address):
-    """Return whether given value is a valid MAC."""
-    m = "[0-9a-f]{2}(:[0-9a-f]{2}){5}$"
-    return (isinstance(address, six.string_types)
-            and re.match(m, address.lower()))
-
-
-def get_auth_strategy():
-    if CONF.authenticate is not None:
-        return 'keystone' if CONF.authenticate else 'noauth'
-    return CONF.auth_strategy
-
-
 def get_valid_macs(data):
     """Get a list of valid MAC's from the introspection data."""
     return [m['mac']
             for m in data.get('all_interfaces', {}).values()
             if m.get('mac')]
+
+
+_INVENTORY_MANDATORY_KEYS = ('memory', 'cpu', 'interfaces')
+
+
+def get_inventory(data, node_info=None):
+    """Get and validate the hardware inventory from introspection data."""
+    inventory = data.get('inventory')
+    # TODO(dtantsur): validate inventory using JSON schema
+    if not inventory:
+        raise Error(_('Hardware inventory is empty or missing'),
+                    data=data, node_info=node_info)
+
+    for key in _INVENTORY_MANDATORY_KEYS:
+        if not inventory.get(key):
+            raise Error(_('Invalid hardware inventory: %s key is missing '
+                          'or empty') % key, data=data, node_info=node_info)
+
+    if not inventory.get('disks'):
+        LOG.info(_LI('No disks were detected in the inventory, assuming this '
+                     'is a disk-less node'), data=data, node_info=node_info)
+        # Make sure the code iterating over it does not fail with a TypeError
+        inventory['disks'] = []
+
+    return inventory
+
+
+def iso_timestamp(timestamp=None, tz=pytz.timezone('utc')):
+    """Return an ISO8601-formatted timestamp (tz: UTC) or None.
+
+    :param timestamp: such as time.time() or None
+    :param tz: timezone
+    :returns: an ISO8601-formatted timestamp, or None
+    """
+    if timestamp is None:
+        return None
+    date = datetime.datetime.fromtimestamp(timestamp, tz=tz)
+    return date.isoformat()
